# manages the layer input and some general fit and scan input

#########################################################################################################
# file structure:                                                                                       #
# create the tab                                                                                        #
# create, modify and delete the layer tab                                                               #
# useful variables:                                                                                    #
# self.layers: list of the layers of the structure [['name', thickness, roughness, composition], ...]   #
# self.minim_list: list of the variables to minimize                                                    #
#########################################################################################################


import sys
from PyQt4 import QtGui, QtCore
import string
from string import atoi,atof

import layerdialog
import stechiometria as ste
import simtab

class Inptab(QtGui.QWidget):
    def __init__(self, parent = None):
        QtGui.QWidget.__init__(self, parent)
        self.setWindowTitle('input tab')

###############################################################
# create page
        self.layerGroup = QtGui.QGroupBox(self.tr('Layers management'))
        self.layerGroup.setMaximumSize(QtCore.QSize(450,60))
        self.addButton = QtGui.QPushButton('ADD')
        self.selectButton = QtGui.QPushButton('Select')
        self.deleteButton = QtGui.QPushButton('Del')
        self.simButton = QtGui.QPushButton('Sim')
        self.questionButton = QtGui.QPushButton('?')
        self.selectCombo = QtGui.QComboBox()
        self.repLayer = QtGui.QLabel('Rep.')
        self.repSpin = QtGui.QSpinBox()
        self.repSpin.setValue(1)
        self.layers = []
        self.minim_list = []
        self.layerGroupLayout = QtGui.QHBoxLayout()
        self.layerGroupLayout.addWidget(self.addButton)
        self.connect(self.addButton, QtCore.SIGNAL('clicked()'), self.modifyDialog)
        self.layerGroupLayout.addWidget(self.selectButton)
        self.connect(self.selectButton, QtCore.SIGNAL('clicked()'), self.showDialog)
        self.layerGroupLayout.addWidget(self.deleteButton)
        self.connect(self.deleteButton, QtCore.SIGNAL('clicked()'), self.deleteDialog)
        self.layerGroupLayout.addWidget(self.selectCombo)
        self.layerGroupLayout.addWidget(self.repLayer)
        self.layerGroupLayout.addWidget(self.repSpin)
        self.layerGroupLayout.addWidget(self.questionButton)
        self.connect(self.questionButton, QtCore.SIGNAL('clicked()'), self.showInformation)
        self.layerGroup.setLayout(self.layerGroupLayout)

        self.inpGroup = QtGui.QGroupBox(self.tr('Input'))
        self.inp_1 = QtGui.QLabel('f0_table')
        self.inp_1_edit = QtGui.QLineEdit('f0_WaasKirf.dat')
        self.inp_2 = QtGui.QLabel('f1f2_table')
        self.inp_2_edit = QtGui.QLineEdit('f1f2_Sasaki.dat')
        self.inp_n = QtGui.QLabel('file')
        self.inp_n_edit = QtGui.QLineEdit()
        self.tabButton = QtGui.QPushButton('?')
        self.inpGroupLayout = QtGui.QGridLayout()
        self.inpGroupLayout.addWidget(self.inp_1, 0,0)
        self.inpGroupLayout.addWidget(self.inp_1_edit, 0,1)
        self.inpGroupLayout.addWidget(self.inp_2, 1,0)
        self.inpGroupLayout.addWidget(self.inp_2_edit, 1,1)
        self.inpGroupLayout.addWidget(self.tabButton, 1,2)
        self.connect(self.tabButton, QtCore.SIGNAL('clicked()'), self.showInformationTables)
        self.inpGroupLayout.addWidget(self.inp_n, 2,0)
        self.inpGroupLayout.addWidget(self.inp_n_edit, 2,1)
        self.inpGroup.setLayout(self.inpGroupLayout)
        
        self.scanGroup = QtGui.QGroupBox(self.tr('Scan general input'))
        self.angle_lab = QtGui.QLabel('Angle factor')
        self.angle_edit = QtGui.QLineEdit('3.14159265/180.0')
        self.norm_lab = QtGui.QLabel('Norm.')
        self.norm_edit = QtGui.QLineEdit('1.0')
        self.noise_lab = QtGui.QLabel('Noise')
        self.noise_edit = QtGui.QLineEdit('0.0')
        self.scanLayout = QtGui.QGridLayout()
        self.scanLayout.addWidget(self.angle_lab, 0,2)
        self.scanLayout.addWidget(self.angle_edit, 0,3)
        self.scanLayout.addWidget(self.norm_lab, 1,0)
        self.scanLayout.addWidget(self.norm_edit, 1,1)
        self.scanLayout.addWidget(self.noise_lab, 1,2)
        self.scanLayout.addWidget(self.noise_edit, 1,3)
        self.scanGroup.setLayout(self.scanLayout)
        
        self.minGroup = QtGui.QGroupBox(self.tr('Fit general input'))
        self.temp_lab = QtGui.QLabel('Annealing')
        self.temp_edit = QtGui.QLineEdit('.05*exp(-0.2*x)')
        self.iter_lab = QtGui.QLabel('maxiters')
        self.iter_edit = QtGui.QLineEdit('1000')
        self.maxref_lab = QtGui.QLabel('Max_refusedcont')
        self.maxref_edit = QtGui.QLineEdit('100')
        self.isthesame_lab = QtGui.QLabel('Max_isthesame')
        self.isthesame_edit = QtGui.QLineEdit('4')
        self.optimize_lab = QtGui.QLabel('Fit method')
        self.optimize_combo = QtGui.QComboBox()
        self.optimize_combo.addItems(['Amoeba', 'Leastsq', 'Both'])
        self.minLayout = QtGui.QGridLayout()
        self.minLayout.addWidget(self.temp_lab, 0,0)
        self.minLayout.addWidget(self.temp_edit, 0,1)
        self.minLayout.addWidget(self.iter_lab, 0,2)
        self.minLayout.addWidget(self.iter_edit, 0,3)
        self.minLayout.addWidget(self.optimize_lab, 0,4)
        self.minLayout.addWidget(self.maxref_lab, 1,0)
        self.minLayout.addWidget(self.maxref_edit, 1,1)
        self.minLayout.addWidget(self.isthesame_lab, 1,2)
        self.minLayout.addWidget(self.isthesame_edit, 1,3)
        self.minLayout.addWidget(self.optimize_combo, 1,4)
        self.minGroup.setLayout(self.minLayout)

        
        
        self.MainLayout = QtGui.QVBoxLayout()
        self.MainLayout.addWidget(self.layerGroup)
        self.MainLayout.addWidget(self.inpGroup)
        self.MainLayout.addWidget(self.scanGroup)
        self.MainLayout.addWidget(self.minGroup)
        
        self.setLayout(self.MainLayout)
        
########################################################################


# Function that permits to modify a layer
# reminder, self.layers = [['layer_name1', thickness1, roughness1, composition1 (element1 partial_density1; el2 pd2; ...)], ...]
        
    def showDialog(self):
#         if no layer is specified yet, it does nothing
        if len(self.layers) == 0 :
            return
        else :
#             otherwise open a dialog as specified in layerdialog.py
            self.dialog = layerdialog.InpDialog()
            msg = str(self.selectCombo.currentText())
            for i in range(len(self.layers)):
#                 finds the index of self.layers that contains the specified layer
                if msg == self.layers[i][0]:
                   a = i
                   break
#                 put the values in the GUI
            self.dialog.inp_1_edit.setText(self.layers[a][0])
            self.dialog.inp_2_edit.setText(self.layers[a][1])
            self.dialog.inp_n_edit.setText(self.layers[a][2])
            self.dialog.inp_3_edit.setText(self.layers[a][3])
            self.dialog.composition_edit.setText(ste.to_st(self.layers[a][4]))
            self.dialog.show()
            if self.dialog.exec_():
#                 when closed, updates self.layers
                msg = [a, self.dialog.inp_1_edit.text()]
                self.layers[a][0] = str(self.dialog.inp_1_edit.text())
                self.layers[a][1] = str(self.dialog.inp_2_edit.text())
                self.layers[a][2] = str(self.dialog.inp_n_edit.text())
                self.layers[a][3] = str(self.dialog.inp_3_edit.text())
                self.layers[a][4] = ste.from_st(str(self.dialog.composition_edit.text()))
                self.selectCombo.setItemText(a, self.dialog.inp_1_edit.text())
#                 if the layer name is modified, updates also the optimization tags
                self.emit(QtCore.SIGNAL('modify_mincombo'),msg)
 
                
# create a new layer           
    def modifyDialog(self):
        self.dialog = layerdialog.InpDialog()
        self.dialog.show()
        if self.dialog.exec_():
            msg = [str(self.dialog.inp_1_edit.text()),str(self.dialog.inp_2_edit.text()),str(self.dialog.inp_n_edit.text()),str(self.dialog.inp_3_edit.text()),ste.from_st(str(self.dialog.composition_edit.text()))]
            self.layers.append(msg)
#             update the combo with layer names
            self.selectCombo.addItem(msg[0])
#             update the optimization tags
            self.emit(QtCore.SIGNAL('dialog_mod'),msg[0])
       
#             delete a layer     
    def deleteDialog(self):
        if len(self.layers) == 0 :
            return
        else :
            msg = self.selectCombo.currentText()
            for i in range(len(self.layers)):
                if msg == self.layers[i][0]:
                   a = i
                   break
            self.selectCombo.removeItem(i)
            del self.layers[i]
            self.emit(QtCore.SIGNAL('dialog_del'),msg)
    
            
    def showInformation(self):
        QtGui.QMessageBox.information(self,
                                "Layers Information",
                                ''' sub: substrate layer, thickness must be set to 0.0
                                
  substrate layer name MUST be 'sub' (without quotes)
  
  other layers: give a name (underscore _ forbidden!), thickness and roughness is in A
  density is in g/cm3
  composition: element_name stoichiometry; ecc...
  
  example for corundum: Al 2;O 3;
  example nichel oxide Ni 1; O 1;
  
  layers must be in correct order, first sub, then to botton to top.
  repetitions is for a multilayer (1 for a single structure)
  
  Push Sim to simulate a reflectivity on your structure.
  ''')
  
    def showInformationTables(self):
        QtGui.QMessageBox.information(self,
                                "Tables Information",
                                '''  f0 tables:    f0_CromerMann.dat
                  f0_EPDL97.dat
                  f0_WaasKirf.dat
 f1f2 tables:  f1f2_EPDL97.dat
                  f1f2_Henke.dat
                  f1f2_Sasaki.dat
                  f1f2_Windt.dat
                  f1f2_asf_Kissel.dat''')
        

if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)
    qb = Inptab()
    qb.show()
    sys.exit(app.exec_())
