# Main window. Contains the plot, the log and the tabs



import sys
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from numpy import arange, sin, cos, pi, array, log, diff
from string import *


import matplotlib
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from matplotlib.figure import Figure

from tabdialog import TabDialog
from logtxt import Logtxt
from propdialog import AtomicPropertiesDialog
from backtab import BackgroundDialog

class MainWindow(QMainWindow):
    def __init__(self):
        import locale
        locale.setlocale(locale.LC_NUMERIC, "C")
        QMainWindow.__init__(self)
        self.setWindowTitle('CARD')
        self.statusBar().showMessage('Ready')
        self.setWindowIcon(QIcon('icons/joker.png'))
        self.main_frame = QWidget()
        plot_frame = QWidget()
        self.dpi = 100
        self.fig = Figure((6.0, 4.0), dpi=self.dpi)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)
        
        self.tabs = TabDialog()
        self.logs = Logtxt()
        self.backdialog = BackgroundDialog()
        
        self.axes = self.fig.add_subplot(111)
        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        PlotGroup = QGroupBox('plot')
        vbox = QVBoxLayout()
        vbox.addWidget(self.canvas)
        vbox.addWidget(self.mpl_toolbar)
        PlotGroup.setLayout(vbox)

        CalcGroup = QGroupBox('calc')
        vbox2 = QVBoxLayout()
        vbox2.addWidget(self.tabs)
        vbox2.addWidget(self.logs)
        CalcGroup.setLayout(vbox2)

        hbox = QHBoxLayout()
        hbox.addWidget(CalcGroup)
        hbox.addWidget(PlotGroup)
        self.main_frame.setLayout(hbox)

        self.setCentralWidget(self.main_frame)
        
        self.menubar = self.menuBar()
        
        # menu File

        save = QAction(QIcon('icons/save.png'), 'Save', self)
        save.setShortcut('Ctrl+S')
        save.setStatusTip('Save File')
        self.connect(save, SIGNAL('triggered()'), self.tabs.selectSave)
        
        saveAs = QAction(QIcon('icons/save.png'), 'Save As', self)
        saveAs.setShortcut('Shift+Ctrl+S')
        saveAs.setStatusTip('Save File With a different name')
        self.connect(saveAs, SIGNAL('triggered()'), self.tabs.saveAs)

        load = QAction(QIcon('icons/Load.png'), 'Load', self)
        load.setShortcut('Ctrl+O')
        load.setStatusTip('Load File')
        self.connect(load, SIGNAL('triggered()'), self.tabs.loadXML)

        
        exit = QAction(QIcon('icons/exit.png'), 'Exit', self)
        exit.setShortcut('Ctrl+Q')
        exit.setStatusTip('Exit application')
        self.connect(exit, SIGNAL('triggered()'), SLOT('close()'))



 
        self.file = self.menubar.addMenu('&File')
        self.file.addAction(save)
        self.file.addAction(saveAs)
        self.file.addAction(load)
        self.file.addAction(exit)
        
        
        
        
        
        
        
        ##################################
        
        #         menu Tools
        
        sh_dial = QAction( 'Atomic Properties', self)
        sh_dial.setShortcut('Ctrl+A')
        sh_dial.setStatusTip('Show different atomic Properties')
        self.connect(sh_dial, SIGNAL('triggered()'), self.showDialog)
        
        sh_bkg = QAction( 'BackGround Subtraction', self)
        sh_bkg.setShortcut('Ctrl+B')
        sh_bkg.setStatusTip('Subctract background from a file')
        self.connect(sh_bkg, SIGNAL('triggered()'), self.showBackDialog)

        plot_fi = QAction( 'Plot RvsPhi', self)
        plot_fi.setShortcut('Ctrl+F')
        plot_fi.setStatusTip('Plot current exp and calculated RvsPhi')
        self.connect(plot_fi, SIGNAL('triggered()'), self.tabs.Plotinpfi)
        
        fit_fi = QAction( 'Fit RvsPhi', self)
        fit_fi.setShortcut('Ctrl+Shift+F')
        fit_fi.setStatusTip('Fit current exp and calculated RvsPhi and plot the results')
        self.connect(fit_fi, SIGNAL('triggered()'), self.tabs.optfi)
        
        plot_e = QAction( 'Plot RvsE', self)
        plot_e.setShortcut('Ctrl+E')
        plot_e.setStatusTip('Plot current exp and calculated RvsE')
        self.connect(plot_e, SIGNAL('triggered()'), self.tabs.Plotinpen)
        
        fit_e = QAction( 'Fit RvsE', self)
        fit_e.setShortcut('Ctrl+Shift+E')
        fit_e.setStatusTip('Fit current exp and calculated RvsE and plot the results')
        self.connect(fit_e, SIGNAL('triggered()'), self.tabs.optre)
        
        self.tools = self.menubar.addMenu('&Tools')
        self.tools.addAction(sh_dial)
        self.tools.addAction(sh_bkg)
        self.tools.addAction(plot_fi)
        self.tools.addAction(fit_fi)
        self.tools.addAction(plot_e)
        self.tools.addAction(fit_e)
        
        
        ############################################
        
#         menu Plot

        plot_dR = QAction( 'derivative', self)
        plot_dR.setShortcut('Ctrl+1')
        plot_dR.setStatusTip('Plot derivative')
        self.connect(plot_dR, SIGNAL('triggered()'), self.deriv)
        
        lin_lin = QAction( 'linlin', self)
        lin_lin.setShortcut('Ctrl+2')
        lin_lin.setStatusTip('Plot linear both axes')
        self.connect(lin_lin, SIGNAL('triggered()'), self.linlin)
        
        log_lin = QAction( 'loglin', self)
        log_lin.setShortcut('Ctrl+3')
        log_lin.setStatusTip('Plot log(x), linear y')
        self.connect(log_lin, SIGNAL('triggered()'), self.loglin)
        
        lin_log = QAction( 'linlog', self)
        lin_log.setShortcut('Ctrl+4')
        lin_log.setStatusTip('Plot linear x, log(y)')
        self.connect(lin_log, SIGNAL('triggered()'), self.linlog)
        
        log_log = QAction( 'loglog', self)
        log_log.setShortcut('Ctrl+5')
        log_log.setStatusTip('Plot log of both axes')
        self.connect(log_log, SIGNAL('triggered()'), self.loglog)
        
        self.plotta = self.menubar.addMenu('&Plot')
        self.plotta.addAction(plot_dR)
        self.plotta.addAction(lin_lin)
        self.plotta.addAction(log_lin)
        self.plotta.addAction(lin_log)
        self.plotta.addAction(log_log)
        
        #########################################
        

        self.connect(self.tabs,SIGNAL("test_func"),self.plotRef)
        self.connect(self.tabs,SIGNAL("sim_func"),self.plotSim)
        self.connect(self.backdialog, SIGNAL('bkg_plot'), self.plotbak)
        self.connect(self.backdialog, SIGNAL('ref_plot'), self.plotbak)
        self.connect(self.backdialog, SIGNAL('bkg_fit'), self.fitbak)
        self.connect(self.backdialog, SIGNAL('ref_fit'), self.plotbak)
        
        #########################################
#         set the status tip for the buttons and the tags
        
        self.tabs.inptab.layerGroup.setStatusTip('Manage your sample structure, push the ? button to know more')
        self.tabs.inptab.addButton.setStatusTip('Add a new layer on top')
        self.tabs.inptab.selectButton.setStatusTip('Modify current layer')
        self.tabs.inptab.deleteButton.setStatusTip('Delete current layer')
        self.tabs.inptab.inpGroup.setStatusTip('General input, check the DATA/ directory for other optical constants')
        self.tabs.inptab.inp_1.setStatusTip('Current f0 table from Dabax')
        self.tabs.inptab.inp_2.setStatusTip('Current f1f2 table from Dabax')
        self.tabs.inptab.tabButton.setStatusTip('Click to know the implemented tables for optical constants.')
        self.tabs.inptab.inp_n.setStatusTip('Current .xml input file, you can modify it by hand of with "save as" (SHIFT+CTRL+s)')
        self.tabs.inptab.scanGroup.setStatusTip('General input for the exp. reflectivity')
#        self.tabs.inptab.weight_lab.setStatusTip('Weight for the exp. reflectivity. Integer if the weight is a column on exp. file')
        self.tabs.inptab.angle_lab.setStatusTip('Angle conversion to radians. Enter a float or a simple expression.')
        self.tabs.inptab.norm_lab.setStatusTip('Normalization (float). R = R*norm')
        self.tabs.inptab.noise_lab.setStatusTip('Noise (float). R = R + noise')
        self.tabs.inptab.minGroup.setStatusTip('General input for the fit. Useless if you are using the Leastsq method')
        self.tabs.inptab.temp_lab.setStatusTip('Annealing parameter. x is the "temperature" parameter')
        self.tabs.inptab.iter_lab.setStatusTip('Stop the fit after selected number iterations (int).')
        self.tabs.inptab.maxref_lab.setStatusTip('Stop the fit after the routine has rejected selected number of local minima (int).')
        self.tabs.inptab.isthesame_lab.setStatusTip('Stop the fit after the routine has fallen selected times on the same local minima (int).')
        self.tabs.inptab.optimize_lab.setStatusTip('Select fit method: Amoeba annealed, Levemberg-Marquardt algorithm or both (first Amoeba)')
        
        self.tabs.minfitab.fileLabel.setStatusTip('Select the exp. input file (no text or comments allowed!)')
        self.tabs.minfitab.outFile.setStatusTip('Select the output file name. Columns: Calc., wavelenghts, angles, exp., weights')
        self.tabs.minfitab.energyLabel.setStatusTip('Put the energy value in eV')
        self.tabs.minfitab.ficol.setStatusTip('Select the column in the exp. file that contains the angles')
        self.tabs.minfitab.rcol.setStatusTip('Select the column in the exp. file that contains the reflectivity')
        self.tabs.minfitab.rebinCheck.setStatusTip('Check to allow the rebin of the angle column with selected number of points.')
        self.tabs.minfitab.toenergy_lab.setStatusTip('Select a range in the angle to give a different weight to the rebin.')
        self.tabs.minfitab.eweight_lab.setStatusTip('Select the weight to give to the range.')
        self.tabs.minfitab.plotFiButton.setStatusTip('Plot the calc. and the exp. reflectivities.')
        self.tabs.minfitab.optGroup.setStatusTip('Set the parameters to optimize. Press ? button to know more.')
        self.tabs.minfitab.optButton.setStatusTip('Run the PPM fit and plot the results. The fit variables will be updated.')
        self.tabs.minfitab.expButton.setStatusTip('Displays the fit results in a new dialog.')
        self.tabs.minfitab.undoButton.setStatusTip('Undo the last changes and return to the values before the fit.')
        self.tabs.minfitab.addButton.setStatusTip('Add a fit variable. Remember the quotes for the variable name.')
        self.tabs.minfitab.modButton.setStatusTip('modify a fit variable. Remember the quotes for the variable name.')
        self.tabs.minfitab.delButton.setStatusTip('Delete a fit variable.')
        
        
        self.tabs.minreftab.fileLabel.setStatusTip('Select the exp. input file (no text or comments allowed!)')
        self.tabs.minreftab.outFile.setStatusTip('Select the output file name. Columns: Calc., wavelenghts, angles, exp., weights')
        self.tabs.minreftab.fiLabel.setStatusTip('Put the angle with arbitrary units (remember to change the value of "angle factor")')
        self.tabs.minreftab.Ecol.setStatusTip('Select the column in the exp. file that contains the energy (in eV)')
        self.tabs.minreftab.rcol.setStatusTip('Select the column in the exp. file that contains the reflectivity')
        self.tabs.minreftab.rebinCheck.setStatusTip('Check to allow the rebin of the energy column with selected number of points.')
        self.tabs.minreftab.toenergy_lab.setStatusTip('Select a range in the energy to give a different weight to the rebin.')
        self.tabs.minreftab.eweight_lab.setStatusTip('Select the weight to give to the range.')
        self.tabs.minreftab.plotRButton.setStatusTip('Plot the calc. and the exp. reflectivities.')
        self.tabs.minreftab.optGroup.setStatusTip('Set the parameters to optimize. Press ? button to know more.')
        self.tabs.minreftab.optButton.setStatusTip('Run the PPM fit and plot the results. The fit variables will be updated.')
        self.tabs.minreftab.expButton.setStatusTip('Displays the fit results in a new dialog.')
        self.tabs.minreftab.undoButton.setStatusTip('Undo the last changes and return to the values before the fit.')
        self.tabs.minreftab.addButton.setStatusTip('Add a fit variable. Remember the quotes for the variable name.')
        self.tabs.minreftab.modButton.setStatusTip('modify a fit variable. Remember the quotes for the variable name.')
        self.tabs.minreftab.delButton.setStatusTip('Delete a fit variable.')
        
        self.tabs.refltab.L_label.setStatusTip('Select the layer where the feffxxxx.dat files will be calculated.')
        self.tabs.refltab.El_label.setStatusTip('Select the edge element (must be present in the layer).')
        self.tabs.refltab.E0_label.setStatusTip('Select the edge energy (in eV).')
        self.tabs.refltab.feffLabel.setStatusTip('Select the source feffxxxx.dat folder.')
        self.tabs.refltab.outLabel.setStatusTip('Select the destination feffxxxx.dat folder.')
        self.tabs.refltab.pathsLabel.setStatusTip('Select the number of paths that will be calculated (e.g. 15 means that the software will calculate up to feff0015.dat).')
        self.tabs.refltab.a_label.setStatusTip('Angle in which the calculation will be performed (remainder).')
        self.tabs.refltab.goButton.setStatusTip('Start the calculation.')
        
        #         connection for the log_file
        self.connect(self.tabs, SIGNAL('add_log'), self.logs.testFunc)
        
    def plotbak(self, msg):
        self.axes.clear()
        self.x_array = array(msg[0])
        self.y_array = array(msg[1])
        self.axes.plot(self.x_array,self.y_array , label='EXP')
        if msg[0][0] < 6.0:
            self.axes.set_xlabel('Angle')
        else:
            self.axes.set_xlabel('Energy (eV)')
        self.axes.set_ylabel('Reflectivity')
        self.axes.legend()
        self.canvas.draw()
    def fitbak(self, msg):
        self.axes.clear()
        self.x_array = array(msg[0])
        self.y_array = array(msg[1])
        self.y2_array = array(msg[2])
        self.axes.plot(self.x_array,self.y_array , label='EXP')
        self.axes.plot(self.x_array,self.y2_array , label='FIT')
        if msg[0][0] < 6.0:
            self.axes.set_xlabel('Angle')
        else:
            self.axes.set_xlabel('Energy (eV)')
        self.axes.set_ylabel('Reflectivity')
        self.axes.legend()
        self.canvas.draw()
        
    def plotRef(self, msg):
        self.axes.clear()
        self.x_array = array(msg[0])
        self.EXP_array = array(msg[1])
        self.CALC_array = array(msg[2])
        self.messaggio = msg[3]
        self.axes.plot(self.x_array,self.EXP_array , label='EXP')
        self.axes.plot(self.x_array,self.CALC_array , label='CALC')
        if self.messaggio == 'fi':
            self.axes.set_xlabel('Angle (deg)')
        elif self.messaggio == 'en':
            self.axes.set_xlabel('Energy (eV)')
        self.axes.set_ylabel('Reflectivity')
        self.axes.legend()
        self.canvas.draw()
    def plotSim(self, msg):
        self.axes.clear()
        self.x_array = array(msg[0])
        self.CALC_array = array(msg[1])
        self.messaggio = msg[2]
        self.axes.plot(self.x_array,self.CALC_array , label='CALC')
        if self.messaggio == 'fi':
            self.axes.set_xlabel('Angle (deg)')
        elif self.messaggio == 'en':
            self.axes.set_xlabel('Energy (eV)')
        self.axes.set_ylabel('Reflectivity')
        self.axes.legend()
        self.canvas.draw()
        
    def deriv(self):
        self.axes.clear()
        try:
            self.axes.plot(self.x_array[1:],(diff(self.EXP_array)/diff(self.x_array)) , label='EXP')
        except:
            pass
        self.axes.plot(self.x_array[1:],(diff(self.CALC_array)/diff(self.x_array)) , label='CALC')
        if self.messaggio == 'fi':
            self.axes.set_xlabel('Angle (deg)')
            self.axes.set_ylabel('dR/dphi')
        elif self.messaggio == 'en':
            self.axes.set_xlabel('Energy (eV)')
            self.axes.set_ylabel('dR/dE')
        self.axes.legend()
        self.canvas.draw()
        
    def linlin(self):
        self.axes.clear()
        try:
            self.axes.plot(self.x_array,self.EXP_array , label='EXP')
        except:
            pass
        self.axes.plot(self.x_array,self.CALC_array , label='CALC')
        if self.messaggio == 'fi':
            self.axes.set_xlabel('Angle (deg)')
        elif self.messaggio == 'en':
            self.axes.set_xlabel('Energy (eV)')
        self.axes.set_ylabel('Reflectivity')
        self.axes.legend()
        self.canvas.draw()
    def loglin(self):
        self.axes.clear()
        try:
            self.axes.plot(log(self.x_array),self.EXP_array , label='EXP')
        except:
            pass
        self.axes.plot(log(self.x_array),self.CALC_array , label='CALC')
        if self.messaggio == 'fi':
            self.axes.set_xlabel('log(Angle (deg))')
        elif self.messaggio == 'en':
            self.axes.set_xlabel('log(Energy (eV))')
        self.axes.set_ylabel('Reflectivity')
        self.axes.legend()
        self.canvas.draw()
    def linlog(self):
        self.axes.clear()
        try:
            self.axes.plot(self.x_array,log(self.EXP_array) , label='EXP')
        except:
            pass
        self.axes.plot(self.x_array,log(self.CALC_array) , label='CALC')
        if self.messaggio == 'fi':
            self.axes.set_xlabel('Angle (deg)')
        elif self.messaggio == 'en':
            self.axes.set_xlabel('Energy (eV)')
        self.axes.set_ylabel('log(Reflectivity)')
        self.axes.legend()
        self.canvas.draw()
    def loglog(self):
        self.axes.clear()
        try:
            self.axes.plot(log(self.x_array),log(self.EXP_array) , label='EXP')
        except:
            pass
        self.axes.plot(log(self.x_array),log(self.CALC_array) , label='CALC')
        if self.messaggio == 'fi':
            self.axes.set_xlabel('log(Angle (deg))')
        elif self.messaggio == 'en':
            self.axes.set_xlabel('log(Energy (eV))')
        self.axes.set_ylabel('log(Reflectivity)')
        self.axes.legend()
        self.canvas.draw()
        
    def showDialog(self):
        self.dialog = AtomicPropertiesDialog()
        self.dialog.show()
    def showBackDialog(self):
        
        self.backdialog.show()
        
        

if __name__ == '__main__':
    app = QApplication(sys.argv)
    main = MainWindow()
    main.show()
    sys.exit(app.exec_())
